@@ -16,7 +16,7 @@
 
 from __future__ import absolute_import
 
-import gtk
+from gi.repository import Gtk
 
 from solfege import abstract
 from solfege import cfg
@@ -51,19 +51,19 @@ class Gui(abstract.LessonbasedGui):
         self.g_music_displayer = mpd.MusicDisplayer()
         self.set_size_request(500, -1)
         self.g_music_displayer.show()
-        self.practise_box.pack_start(self.g_music_displayer)
+        self.practise_box.pack_start(self.g_music_displayer, True, True, 0)
         ###############
         # action_area #
         ###############
         self.g_partbox = gu.bHBox(self.practise_box, False)
-        self.g_go_back = gtk.Button(stock='gtk-go-back')
+        self.g_go_back = Gtk.Button(stock='gtk-go-back')
         self.g_go_back.connect('clicked', self.select_previous)
         self.g_go_back.show()
-        self.action_area.pack_start(self.g_go_back, False)
-        self.g_go_forward = gtk.Button(stock='gtk-go-forward')
+        self.action_area.pack_start(self.g_go_back, False, False, 0)
+        self.g_go_forward = Gtk.Button(stock='gtk-go-forward')
         self.g_go_forward.show()
         self.g_go_forward.connect('clicked', self.select_next)
-        self.action_area.pack_start(self.g_go_forward, False)
+        self.action_area.pack_start(self.g_go_forward, False, False, 0)
         self.g_play = gu.bButton(self.action_area, _("_Play the whole music"),
                                  self.play)
         self.g_show = gu.bButton(self.action_area, _("_Show"), self.show_answer)
@@ -104,8 +104,6 @@ class Gui(abstract.LessonbasedGui):
         # see Teacher docstring.
         if self.m_t.q_status != self.QSTATUS_NEW:
             return
-        if not self.m_t.m_P:
-            return
         try:
             self.m_t.m_P.play_question()
         except Exception, e:
@@ -117,8 +115,6 @@ class Gui(abstract.LessonbasedGui):
         # see Teacher docstring.
         if self.m_t.q_status != self.QSTATUS_NEW:
             return
-        if not self.m_t.m_P:
-            return
         try:
             self.g_music_displayer.display(self.m_t.m_P.get_music(),
                             self.get_int('config/feta_font_size=20'))
@@ -157,14 +153,14 @@ class Gui(abstract.LessonbasedGui):
                 btn = self.create_pixmap_button()
                 btn.connect('clicked', f, None, None)
                 btn.show()
-                self.g_partbox.pack_start(btn)
+                self.g_partbox.pack_start(btn, True, True, 0)
                 return
             tmp = [Rat(0, 1)] + v + [Rat(2**30, 1)]
             for i in range(len(tmp) - 1):
                 btn = self.create_pixmap_button()
                 btn.show()
                 btn.connect('clicked', f, tmp[i], tmp[i+1])
-                self.g_partbox.pack_start(btn)
+                self.g_partbox.pack_start(btn, True, True, 0)
         # q_status is QSTATUS_NO if the question is invalid (from the lessonfile)
         if self.m_t.q_status == self.QSTATUS_NO:
             self.g_partbox.set_sensitive(False)
@@ -223,13 +219,14 @@ class Gui(abstract.LessonbasedGui):
             self.g_show.set_sensitive(True)
         self._update()
     def create_pixmap_button(self):
-        im = gtk.Image()
-        im.set_from_stock("solfege-rhythm-c4", gtk.ICON_SIZE_LARGE_TOOLBAR)
+        im = Gtk.Image()
+        im.set_from_stock("solfege-rhythm-c4", Gtk.IconSize.LARGE_TOOLBAR)
         im.show()
-        btn = gtk.Button()
+        btn = Gtk.Button()
         btn.add(im)
         return btn
     def on_start_practise(self):
+        super(Gui, self).on_start_practise()
         self.update_gui_after_lessonfile_change()
         self.g_play.grab_focus()
 
