@@ -19,7 +19,8 @@ from __future__ import absolute_import
 import logging
 import random
 
-import gtk
+from gi.repository import Gtk
+from gi.repository import Gdk
 
 from solfege import abstract
 from solfege import cfg
@@ -90,7 +91,6 @@ class Teacher(abstract.Teacher):
                 self.m_statistics.add_correct(unicode(bpm))
             self.q_status = self.QSTATUS_SOLVED
             soundcard.synth.stop()
-            solfege.app.play_happy_sound()
             return 1
         else:
             if self.q_status == self.QSTATUS_NEW:
@@ -116,20 +116,20 @@ class Gui(abstract.Gui):
         self.m_buttons = []
         vbox = gu.bVBox(self.practise_box, False)
         for s, e in ((0, 9), (9, 18), (18, 26), (26, 34)):
-            box = gtk.HBox()
-            vbox.pack_start(box)
+            box = Gtk.HBox()
+            vbox.pack_start(box, True, True, 0)
             for i in range(s, e):
                 bpm = self.m_t.m_bpms[i]
-                button = gtk.Button(str(bpm))
-                box.pack_start(button)
+                button = Gtk.Button(str(bpm))
+                box.pack_start(button, True, True, 0)
                 button.connect('clicked', self.on_click)
-                button.connect('event', self.on_event)
-                button.set_data('bpm', bpm)
+                button.connect('button-release-event', self.on_event)
+                button.m_bpm = bpm
                 self.m_buttons.append(button)
 
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
-        self.practise_box.pack_start(self.g_flashbar, False)
+        self.practise_box.pack_start(self.g_flashbar, False, False, 0)
 
         self.std_buttons_add(
             ('new', self.on_new),
@@ -143,15 +143,15 @@ class Gui(abstract.Gui):
         self.g_statview = statisticsviewer.StatisticsViewer(self.m_t.m_statistics, _('Bpm'))
         self.g_statview.show()
         self.g_notebook.append_page(self.g_statview,
-                                    gtk.Label(_("Statistics")))
+                                    Gtk.Label(label=_("Statistics")))
         ########
         # init #
         ########
         self.g_notebook.get_nth_page(1).hide()
         self.update_buttons()
     def on_event(self, button, event):
-        if event.type == gtk.gdk.BUTTON_RELEASE and event.button == 3:
-            self.m_t.toggle_active(button.get_data('bpm'))
+        if event.button == 3:
+            self.m_t.toggle_active(button.m_bpm)
             self.update_buttons()
     def on_level_change(self, adjustment):
         self.set_int('level', adjustment.value)
@@ -178,20 +178,20 @@ class Gui(abstract.Gui):
     def update_buttons(self):
         v = self.m_t.get_possible_bpms()
         for b in self.m_buttons:
-            if b.get_data('bpm') in v:
+            if b.m_bpm in v:
                 b.get_children()[0].set_name("BpmActiveLabel")
             else:
                 b.get_children()[0].set_name("BpmInactiveLabel")
     def on_click(self, _o):
         if self.m_t.q_status in (self.QSTATUS_SOLVED, self.QSTATUS_GIVE_UP):
             return
-        if _o.get_data('bpm') not in self.m_t.get_possible_bpms():
+        if _o.m_bpm not in self.m_t.get_possible_bpms():
             return
         if self.m_t.q_status == self.QSTATUS_NO:
             self.g_flashbar.flash(_("Click 'New tempo' to begin."))
             return
-        if self.m_t.guess_answer(_o.get_data('bpm')):
-            self.g_flashbar.flash(_("Correct, it is %i") % _o.get_data('bpm'))
+        if self.m_t.guess_answer(_o.m_bpm):
+            self.g_flashbar.flash(_("Correct, it is %i") % _o.m_bpm)
             self.std_buttons_answer_correct()
         else:
             self.g_flashbar.flash(_("Wrong"))
