@@ -17,8 +17,9 @@
 from __future__ import absolute_import
 from __future__ import division
 
-import gobject
-import gtk
+from gi.repository import GObject
+from gi.repository import Gdk
+from gi.repository import Gtk
 
 from solfege import abstract
 from solfege import const
@@ -52,12 +53,12 @@ class Teacher(abstract.Teacher, abstract.RhythmAddOnClass):
             self.q_status = self.QSTATUS_WRONG
 
 
-class RhythmViewer(gtk.Frame):
+class RhythmViewer(Gtk.Frame):
     def __init__(self, parent):
-        gtk.Frame.__init__(self)
-        self.set_shadow_type(gtk.SHADOW_IN)
+        Gtk.Frame.__init__(self)
+        self.set_shadow_type(Gtk.ShadowType.IN)
         self.g_parent = parent
-        self.g_box = gtk.HBox()
+        self.g_box = Gtk.HBox(False, 0)
         self.g_box.show()
         self.g_box.set_spacing(gu.PAD_SMALL)
         self.g_box.set_border_width(gu.PAD)
@@ -78,10 +79,10 @@ class RhythmViewer(gtk.Frame):
         create those |__| that represents one beat
         """
         if self.__timeout:
-            gobject.source_remove(self.__timeout)
+            GObject.source_remove(self.__timeout)
         self.clear()
         for x in range(self.m_num_beats):
-            self.g_box.pack_start(gu.create_png_image('holder'), False)
+            self.g_box.pack_start(gu.create_png_image('holder'), False, False, 0)
         self.m_data = []
     def clear_wrong_part(self):
         """When the user have answered the question, this method is used
@@ -98,18 +99,18 @@ class RhythmViewer(gtk.Frame):
             self.g_box.get_children()[n].destroy()
         self.m_data = self.m_data[:n]
         for x in range(n, self.m_num_beats):
-            self.g_box.pack_start(gu.create_png_image('holder'), False)
+            self.g_box.pack_start(gu.create_png_image('holder'), False, False, 0)
     def add_rhythm_element(self, i):
         assert len(self.m_data) <= self.m_num_beats
         if len(self.g_box.get_children()) >= self.m_num_beats:
             self.g_box.get_children()[self.m_num_beats-1].destroy()
-        vbox = gtk.VBox()
+        vbox = Gtk.VBox(False, 0)
         vbox.show()
         im = gu.create_rhythm_image(const.RHYTHMS[i])
-        vbox.pack_start(im)
-        vbox.pack_start(gu.create_png_image('rhythm-wrong'), False, False)
+        vbox.pack_start(im, True, True, 0)
+        vbox.pack_start(gu.create_png_image('rhythm-wrong'), False, False, 0)
         vbox.get_children()[-1].hide()
-        self.g_box.pack_start(vbox, False)
+        self.g_box.pack_start(vbox, False, False, 0)
         self.g_box.reorder_child(vbox, len(self.m_data))
         self.m_data.append(i)
     def backspace(self):
@@ -119,7 +120,7 @@ class RhythmViewer(gtk.Frame):
                 self.g_face.destroy()
                 self.g_face = None
             self.g_box.get_children()[len(self.m_data)-1].destroy()
-            self.g_box.pack_start(gu.create_png_image('holder'), False)
+            self.g_box.pack_start(gu.create_png_image('holder'), False, False, 0)
             del self.m_data[-1]
     def mark_wrong(self, idx):
         """
@@ -133,44 +134,44 @@ class RhythmViewer(gtk.Frame):
     def sad_face(self):
         l = gu.HarmonicProgressionLabel(_("Wrong"))
         l.show()
-        self.g_box.pack_start(l, False)
-        self.g_face = gtk.EventBox()
+        self.g_box.pack_start(l, False, False, 0)
+        self.g_face = Gtk.EventBox()
         self.g_face.connect('button_press_event', self.on_sadface_event)
         self.g_face.show()
-        im = gtk.Image()
-        im.set_from_stock('solfege-sadface', gtk.ICON_SIZE_LARGE_TOOLBAR)
+        im = Gtk.Image()
+        im.set_from_stock('solfege-sadface', Gtk.IconSize.LARGE_TOOLBAR)
         im.show()
         self.g_face.add(im)
-        self.g_box.pack_start(self.g_face, False)
+        self.g_box.pack_start(self.g_face, False, False, 0)
     def happy_face(self):
         l = gu.HarmonicProgressionLabel(_("Correct"))
         l.show()
-        self.g_box.pack_start(l, False)
-        self.g_face = gtk.EventBox()
+        self.g_box.pack_start(l, False, False, 0)
+        self.g_face = Gtk.EventBox()
         self.g_face.connect('button_press_event', self.on_happyface_event)
         self.g_face.show()
-        im = gtk.Image()
-        im.set_from_stock('solfege-happyface', gtk.ICON_SIZE_LARGE_TOOLBAR)
+        im = Gtk.Image()
+        im.set_from_stock('solfege-happyface', Gtk.IconSize.LARGE_TOOLBAR)
         im.show()
         self.g_face.add(im)
-        self.g_box.pack_start(self.g_face, False)
+        self.g_box.pack_start(self.g_face, False, False, 0)
     def on_happyface_event(self, obj, event):
-        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
+        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
             self.g_parent.new_question()
     def on_sadface_event(self, obj, event):
-        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
+        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
             self.clear_wrong_part()
     def flash(self, s):
         self.clear()
-        l = gtk.Label(s)
+        l = Gtk.Label(label=s)
         l.set_name("Feedback")
         l.set_alignment(0.0, 0.5)
         l.show()
-        self.g_box.pack_start(l, True, True)
-        lx, ly = l.size_request()
-        sx, sy = self.g_box.size_request()
-        self.g_box.set_size_request(max(lx + gu.PAD * 2, sx), max(ly + gu.PAD * 2, sy))
-        self.__timeout = gobject.timeout_add(2000, self.unflash)
+        self.g_box.pack_start(l, True, True, 0)
+        self.g_box.set_size_request(
+            max(l.size_request().width + gu.PAD * 2, self.g_box.size_request().width),
+            max(l.size_request().height + gu.PAD * 2, self.g_box.size_request().height))
+        self.__timeout = GObject.timeout_add(2000, self.unflash)
     def unflash(self, *v):
         self.__timeout = None
         self.clear()
@@ -181,21 +182,20 @@ class Gui(abstract.Gui, abstract.RhythmAddOnGuiClass):
         abstract.Gui.__init__(self, teacher)
         self.m_key_bindings = {'backspace_ak': self.on_backspace}
         self.g_answer_box = gu.NewLineBox()
-        self.practise_box.pack_start(self.g_answer_box, False)
+        self.practise_box.pack_start(self.g_answer_box, False, False, 0)
         #-------
         hbox = gu.bHBox(self.practise_box)
-        b = gtk.Button(_("Play"))
+        b = Gtk.Button(_("Play"))
         b.show()
         b.connect('clicked', self.play_users_answer)
-        hbox.pack_start(b, False, True)
-        self.practise_box.pack_start(gtk.HBox(), False,
+        hbox.pack_start(b, False, True, 0)
+        self.practise_box.pack_start(Gtk.HBox(False, 0), False, False,
                                      padding=gu.PAD_SMALL)
         self.g_rhythm_viewer = RhythmViewer(self)
         #FIXME the value 52 is dependant on the theme used
         self.g_rhythm_viewer.set_size_request(-1, 52)
         self.g_rhythm_viewer.create_holders()
-        hbox.pack_start(self.g_rhythm_viewer)
-        #self.practise_box.pack_start(self.g_rhythm_viewer, False)
+        hbox.pack_start(self.g_rhythm_viewer, True, True, 0)
 
         # action area
         self.std_buttons_add(
@@ -210,32 +210,32 @@ class Gui(abstract.Gui, abstract.RhythmAddOnGuiClass):
         ##############
         self.add_select_elements_gui()
         #--------
-        self.config_box.pack_start(gtk.HBox(), False,
+        self.config_box.pack_start(Gtk.HBox(False, 0), False, False,
                                    padding=gu.PAD_SMALL)
         self.add_select_num_beats_gui()
         #-----
-        self.config_box.pack_start(gtk.HBox(), False,
+        self.config_box.pack_start(Gtk.HBox(False, 0), False, False,
                                    padding=gu.PAD_SMALL)
         #------
         hbox = gu.bHBox(self.config_box, False)
         hbox.set_spacing(gu.PAD_SMALL)
         hbox.pack_start(gu.nCheckButton(self.m_exname,
                  "not_start_with_rest",
-                 _("Don't start the question with a rest")), False)
-        sep = gtk.VSeparator()
-        hbox.pack_start(sep, False)
-        hbox.pack_start(gtk.Label(_("Beats per minute:")), False)
+                 _("Don't start the question with a rest")), False, False, 0)
+        sep = Gtk.VSeparator()
+        hbox.pack_start(sep, False, False, 0)
+        hbox.pack_start(Gtk.Label(_("Beats per minute:")), False, False, 0)
         spin = gu.nSpinButton(self.m_exname, 'bpm',
-                 gtk.Adjustment(60, 20, 240, 1, 10))
-        hbox.pack_start(spin, False)
+                 Gtk.Adjustment(60, 20, 240, 1, 10))
+        hbox.pack_start(spin, False, False, 0)
         self._add_auto_new_question_gui(self.config_box)
         self.config_box.show_all()
     def pngbutton(self, i):
         "used by the constructor"
-        btn = gtk.Button()
+        btn = Gtk.Button()
         if i > len(const.RHYTHMS):
-            im = gtk.Image()
-            im.set_from_stock("gtk-missing-image", gtk.ICON_SIZE_LARGE_TOOLBAR)
+            im = Gtk.Image()
+            im.set_from_stock("gtk-missing-image", Gtk.IconSize.LARGE_TOOLBAR)
             im.show()
             btn.add(im)
         else:
