@@ -19,7 +19,7 @@ from __future__ import absolute_import
 import logging
 import time
 
-import gtk
+from gi.repository import Gtk
 
 from solfege import abstract
 from solfege import gu
@@ -132,7 +132,7 @@ class Gui(abstract.LessonbasedGui):
         abstract.Gui.__init__(self, teacher)
         #
         self.g_music_displayer = mpd.MusicDisplayer()
-        self.practise_box.pack_start(self.g_music_displayer, False)
+        self.practise_box.pack_start(self.g_music_displayer, False, False, 0)
         #
         self.g_tap = gu.bButton(self.practise_box, _("Tap here"), self.on_tap)
         self.std_buttons_add(
@@ -144,20 +144,20 @@ class Gui(abstract.LessonbasedGui):
         # Flashbar
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
-        self.practise_box.pack_start(self.g_flashbar, False)
+        self.practise_box.pack_start(self.g_flashbar, False, False, 0)
         # Config box
-        label = gtk.Label(_("Accuracy required:"))
+        label = Gtk.Label(label=_("Accuracy required:"))
         self.config_box_sizegroup.add_widget(label)
         label.set_alignment(1.0, 0.5)
         spin = gu.nSpinButton(self.m_exname, 'accuracy',
-                              gtk.Adjustment(0, 0, 2, 0.01, 0.05))
+                              Gtk.Adjustment(0, 0, 2, 0.01, 0.05))
         spin.set_tooltip_text("See bug report #93 (http://bugs.solfege.org/93) and add suggested values to the bug report.")
         spin.set_digits(2)
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         hbox.set_spacing(gu.hig.SPACE_SMALL)
-        hbox.pack_start(label, False)
-        hbox.pack_start(spin, False)
-        self.config_box.pack_start(hbox, False)
+        hbox.pack_start(label, False, False, 0)
+        hbox.pack_start(spin, False, False, 0)
+        self.config_box.pack_start(hbox, False, False, 0)
         hbox.show_all()
     def on_new_question(self, widget=None):
         def exception_cleanup():
