@@ -28,7 +28,7 @@ import subprocess
 import sys
 import textwrap
 
-import gobject
+from gi.repository import GObject
 
 from solfege import cfg
 from solfege import dataparser
@@ -140,49 +140,6 @@ _keys_to_interval = {
                 10: 'M6', # c -> ais
             }
 
-keywords = (
-    'harmonic',
-    'melodic',
-    'progression',
-    'normal',
-    'voice',
-    'rvoice',
-    'satb',
-    'rhythm',
-    'cmdline',
-    'wavfile',
-    'midifile',
-    'mma',
-    'horiz',
-    'vertic',
-    'accidentals',
-    'key',
-    'semitones',
-    'atonal',
-    'play',
-    'show',
-    # elembuilder:
-    'auto',
-    # Used by the rhythm module:
-    'newline',
-    'd1', 'p1', 'a1',
-    'd2', 'm2', 'M2', 'a2',
-    'd3', 'm3', 'M3', 'a3',
-    'd4', 'p4', 'a4',
-    'd5', 'p5', 'a5',
-    'd6', 'm6', 'M6', 'a6',
-    'd7', 'm7', 'M7', 'a7',
-    'd8', 'p8', 'a8',
-    'd9', 'm9', 'M9', 'a9',
-    'd10', 'm10', 'M10', 'a10',
-    # clefs:
-    'treble', 'violin', 'french',
-    'subbass', 'bass', 'varbaritone',
-    'baritone', 'tenor', 'alto', 'mezzosoprano', 'soprano',
-)
-predef = {}
-for n in keywords:
-    predef[n] = n
 
 def load_text(parser, filename):
     """
@@ -335,6 +292,7 @@ class _Header(dict):
                            ('have_music_displayer', False),
                            ('enable_right_click', True),
                            ('disable_unused_intervals', True),
+                           ('statistics_matrices', 'enabled'),
                            ):
             if key not in self:
                 self[key] = value
@@ -351,7 +309,8 @@ class _Header(dict):
         if name in self:
             return self[name]
         return istr("")
-
+    def __setattr__(self, name, value):
+        self[name] = value
 
 class MusicBaseClass(object):
     """
@@ -1020,7 +979,7 @@ def parse_test_def(s):
 
 
 class LessonfileCommon(object):
-    def __init__(self, module_predefs={}):
+    def __init__(self, module_predefs=None, header_defaults=None):
         self.m_prev_question = None
         # This variable stores the directory the lesson file is located in.
         # We need this to we can find other files relative to this file.
@@ -1029,7 +988,14 @@ class LessonfileCommon(object):
         self._idx = None
         self.m_filename = "<STRING>"
         self.m_translation_re = re.compile("(?P<varname>\w+)\[(?P<lang>[\w_+]+)\]")
-        self.m_module_predefs = module_predefs
+        if module_predefs is None:
+            self.m_module_predefs = {}
+        else:
+            self.m_module_predefs = module_predefs
+        if header_defaults is None:
+            self.m_header_defaults = {}
+        else:
+            self.m_header_defaults = header_defaults
     def parse_file(self, filename):
         """Parse the file named filename. Set these variables:
         self.header     a Header instance
@@ -1037,6 +1003,9 @@ class LessonfileCommon(object):
         """
         self.m_location = os.path.split(uri_expand(filename))[0]
         self.m_filename = filename
+        # We open and read the file without using the codecs module
+        # because the lexer class will check for a coding tag in
+        # the lesson file before decoding it.
         s = open(uri_expand(filename), 'rU').read()
         self.parse_string(s, really_filename=filename)
     def parse_string(self, s, really_filename=None):
@@ -1094,11 +1063,12 @@ class LessonfileCommon(object):
             self.header = _Header(self.blocklists['header'][0])
         else:
             self.header = _Header({})
+        self.header.update(self.m_header_defaults)
 
 
 class QuestionsLessonfile(LessonfileCommon):
-    def __init__(self, module_predefs={}):
-        LessonfileCommon.__init__(self, module_predefs)
+    def __init__(self, module_predefs=None, header_defaults=None):
+        LessonfileCommon.__init__(self, module_predefs, header_defaults)
         self.m_discards = []
     def select_random_question(self):
         """
@@ -1301,7 +1271,7 @@ class QuestionsLessonfile(LessonfileCommon):
     def get_score(self, varname='music'):
         """
         Return the elems.Score object of the music in the variable named
-        by VARNAME. 
+        by VARNAME.
         """
         assert self._idx is not None
         musicobj = self.m_questions[self._idx][varname]
@@ -1899,7 +1869,7 @@ class InfoCache(object):
                     print "all files parsed:", time.time() - start_time
                     pt.Identifier.check_ns = True
                     return False
-            gobject.idle_add(on_idle_parse)
+            GObject.idle_add(on_idle_parse)
         else:
             list(self.iter_parse_all_files())
     def update_modified_files(self):
@@ -1935,7 +1905,7 @@ class InfoCache(object):
         """
         Put the data from the lesson files below ~/.solfege/exercises/
         in the cache and yield the filename.
-        
+
         If only_user_collection=True it will only iterate the lesson files
         in .solfege/exercises/user/lesson-files
         """
@@ -1962,7 +1932,7 @@ def parse_lesson_file_header(filename):
 
     Return None if we find no header block.
     """
-    r = re.compile("\\header\s*{.*?}", re.MULTILINE|re.DOTALL)
+    r = re.compile("header\s*{.*?}", re.MULTILINE|re.DOTALL)
     # We cannot read the whole file, since we don't know what the user
     # have placed in the directory. It could be a whole DVD iso image.
     # The actual size we read, 40k, is mentioned in the user manual,
@@ -1980,6 +1950,6 @@ def parse_lesson_file_header(filename):
     except dataparser.DataparserException:
         return None
     finally:
-        pt.Identifier.check_ns = check_ns    
+        pt.Identifier.check_ns = check_ns
     return p
 
