@@ -1,6 +1,6 @@
 # -*- coding: iso-8859-1 -*-
 # GNU Solfege - free ear training software
-# Copyright (C) 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,18 +21,17 @@ from __future__ import division
 import os
 import sys
 
-import pygtk
-pygtk.require("2.0")
-import gtk
+import gi
+pyGtk.require("2.0")
+from gi.repository import Gtk
 
 from solfege import mpd
 from solfege import gu
 from solfege import lessonfile
 from solfege import dataparser
-from solfege import htmlwidget
 from solfege import stock
 
-gtk.stock_add([('solfege-notehead', _("Add noteheads"), 0, 0, ''),
+Gtk.stock_add([('solfege-notehead', _("Add noteheads"), 0, 0, ''),
                ('solfege-sharp', _("Add sharps"), 0, 0, ''),
                ('solfege-double-sharp', _("Add double-sharps"), 0, 0, ''),
                ('solfege-natural', _("Remove accidentals"), 0, 0, ''),
@@ -42,25 +41,25 @@ gtk.stock_add([('solfege-notehead', _("Add noteheads"), 0, 0, ''),
                ])
 app_version = "0.1.4"
 
-class HelpWindow(gtk.Window):
+class HelpWindow(Gtk.Window):
     def __init__(self, parent):
-        gtk.Window.__init__(self)
+        Gtk.Window.__init__(self)
         self.set_title(_("GNU Solfege lesson file editor") )
         self.set_default_size(400, 400)
         self.g_parent = parent
-        self.vbox = gtk.VBox()
+        self.vbox = Gtk.VBox()
         self.vbox.set_spacing(8)
         self.add(self.vbox)
         self.connect('delete_event', self.delete_cb)
         self.g_htmlwidget = htmlwidget.HtmlWidget(None, None)
-        self.vbox.pack_start(self.g_htmlwidget)
-        self.vbox.pack_start(gtk.HSeparator(), False)
-        bbox = gtk.HButtonBox()
+        self.vbox.pack_start(self.g_htmlwidget, True, True, 0)
+        self.vbox.pack_start(Gtk.HSeparator(), False)
+        bbox = Gtk.HButtonBox()
         bbox.set_border_width(8)
         self.vbox.pack_start(bbox, False)
-        b = gtk.Button(stock=gtk.STOCK_CLOSE)
+        b = Gtk.Button(stock=Gtk.STOCK_CLOSE)
         b.connect('clicked', self.close_cb)
-        bbox.pack_start(b)
+        bbox.pack_start(b, True, True, 0)
         self.show_all()
         self.set_focus(b)
     def source(self, html):
@@ -74,25 +73,25 @@ class HelpWindow(gtk.Window):
 
 window_actions = [
     ('FileMenu', None, _('_File')),
-    ('NewLessonfile', gtk.STOCK_NEW, None, None, 'new file', 'file_new_cb'),
-    ('Open', gtk.STOCK_OPEN, None, None, 'Open lesson file', 'file_open_cb'),
-    ('Save', gtk.STOCK_SAVE, None, None, 'Save the lesson file', 'file_save_cb'),
-    ('SaveAs', gtk.STOCK_SAVE_AS, None, '<shift><ctrl>s', 'Save the lesson file with a new name', 'file_save_as_cb'),
-    ('Quit', gtk.STOCK_QUIT, None, None, 'Quit program', 'quit_cb'),
+    ('NewLessonfile', Gtk.STOCK_NEW, None, None, 'new file', 'file_new_cb'),
+    ('Open', Gtk.STOCK_OPEN, None, None, 'Open lesson file', 'file_open_cb'),
+    ('Save', Gtk.STOCK_SAVE, None, None, 'Save the lesson file', 'file_save_cb'),
+    ('SaveAs', Gtk.STOCK_SAVE_AS, None, '<shift><ctrl>s', 'Save the lesson file with a new name', 'file_save_as_cb'),
+    ('Quit', Gtk.STOCK_QUIT, None, None, 'Quit program', 'quit_cb'),
     ('HelpMenu', None, _('_Help')),
-    ('HelpHelp', gtk.STOCK_HELP, None, None, None, 'help_cb'),
+    ('HelpHelp', Gtk.STOCK_HELP, None, None, None, 'help_cb'),
     ('HelpAbout', None, _('_About'), '', '', 'about_cb'),
 ]
 lessonfile_actions = [
-    ('GotoFirstQuestion', gtk.STOCK_GOTO_FIRST, None, None,
+    ('GotoFirstQuestion', Gtk.STOCK_GOTO_FIRST, None, None,
      _('Go to the first question'), 'goto_first_question_cb'),
-    ('GoBackQuestion', gtk.STOCK_GO_BACK, None, None,
+    ('GoBackQuestion', Gtk.STOCK_GO_BACK, None, None,
      _('Go to the previous question'), 'go_back_question_cb'),
-    ('GoForwardQuestion', gtk.STOCK_GO_FORWARD, None, None,
+    ('GoForwardQuestion', Gtk.STOCK_GO_FORWARD, None, None,
      _('Go to the next question'), 'go_forward_question_cb'),
-    ('GotoLastQuestion', gtk.STOCK_GOTO_LAST, None, None,
+    ('GotoLastQuestion', Gtk.STOCK_GOTO_LAST, None, None,
      _('Go to the last question'), 'goto_last_question_cb'),
-    ('NewQuestion', gtk.STOCK_ADD, None, None,
+    ('NewQuestion', Gtk.STOCK_ADD, None, None,
      _('Add a new question'), 'new_question_cb'),
     ('NoteheadCursor', 'solfege-notehead', None, None,
      _('Add noteheads'), 'select_cursor_notehead_cb'),
@@ -164,29 +163,29 @@ class EditorLessonfile(object):
         self._idx = 0
 
 
-class MainWin(gtk.Window):
+class MainWin(Gtk.Window):
     def __init__(self, datadir):
-        gtk.Window.__init__(self)
+        Gtk.Window.__init__(self)
         self.icons = stock.EditorIconFactory(self, datadir)
-        self.connect('destroy', lambda w: gtk.main_quit())
+        self.connect('destroy', lambda w: Gtk.main_quit())
         self.g_help_window = None
         # toplevel_vbox:
         #   -menubar
         #   -toolbar
         #   -notebook
         #   -statusbar
-        self.toplevel_vbox = gtk.VBox()
+        self.toplevel_vbox = Gtk.VBox()
         self.add(self.toplevel_vbox)
         self.create_menu_and_toolbar()
-        self.g_notebook = gtk.Notebook()
-        self.toplevel_vbox.pack_start(self.g_notebook)
-        self.vbox = gtk.VBox()
-        self.toplevel_vbox.pack_start(self.vbox)
+        self.g_notebook = Gtk.Notebook()
+        self.toplevel_vbox.pack_start(self.g_notebook, True, True, 0)
+        self.vbox = Gtk.VBox()
+        self.toplevel_vbox.pack_start(self.vbox, True, True, 0)
         self.create_mainwin_ui()
         self.show_all()
     def create_mainwin_ui(self):
         qbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(qbox, gtk.Label(_("Questions")))
+        self.g_notebook.append_page(qbox, Gtk.Label(label=_("Questions")))
         gu.bLabel(qbox, _("Enter new chords using the mouse"), False, False)
         hbox = gu.bHBox(qbox, False, False)
         self.g_displayer = mpd.musicdisplayer.ChordEditor()
@@ -196,42 +195,42 @@ class MainWin(gtk.Window):
         hbox.pack_start(self.g_displayer, False)
         gu.bLabel(hbox, "")
         ##
-        self.g_question_name = gtk.Entry()
-        qbox.pack_start(gu.hig_label_widget(_("Question title:"), self.g_question_name, None), False)
-        self.g_navinfo = gtk.Label("")
+        self.g_question_name = Gtk.Entry()
+        qbox.pack_start(gu.hig_label_widget(_("Question title:", True, True, 0), self.g_question_name, None), False)
+        self.g_navinfo = Gtk.Label(label="")
         qbox.pack_start(self.g_navinfo, False)
 
         ##
         self.m_P = EditorLessonfile()
-        cvbox = gtk.VBox()
-        self.g_notebook.append_page(cvbox, gtk.Label(_("Lessonfile header")))
+        cvbox = Gtk.VBox()
+        self.g_notebook.append_page(cvbox, Gtk.Label(label=_("Lessonfile header")))
         ## Header section
-        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        self.g_title = gtk.Entry()
-        cvbox.pack_start(gu.hig_label_widget(_("File title:"), self.g_title, 
+        sizegroup = Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL)
+        self.g_title = Gtk.Entry()
+        cvbox.pack_start(gu.hig_label_widget(_("File title:", True, True, 0), self.g_title,
                         sizegroup))
-        self.g_content_chord = gtk.RadioButton(None, "chord")
-        self.g_content_chord_voicing = gtk.RadioButton(self.g_content_chord, "chord-voicing")
-        self.g_content_idbyname = gtk.RadioButton(self.g_content_chord, "id-by-name")
-        box = gtk.HBox()
-        box.pack_start(self.g_content_chord)
-        box.pack_start(self.g_content_chord_voicing)
-        box.pack_start(self.g_content_idbyname)
-        cvbox.pack_start(gu.hig_label_widget(_("Content:"), box, sizegroup))
-        self.g_random_transpose = gtk.Entry()
-        cvbox.pack_start(gu.hig_label_widget(_("Random transpose:"), 
+        self.g_content_chord = Gtk.RadioButton(None, "chord")
+        self.g_content_chord_voicing = Gtk.RadioButton(self.g_content_chord, "chord-voicing")
+        self.g_content_idbyname = Gtk.RadioButton(self.g_content_chord, "id-by-name")
+        box = Gtk.HBox()
+        box.pack_start(self.g_content_chord, True, True, 0)
+        box.pack_start(self.g_content_chord_voicing, True, True, 0)
+        box.pack_start(self.g_content_idbyname, True, True, 0)
+        cvbox.pack_start(gu.hig_label_widget(_("Content:", True, True, 0), box, sizegroup))
+        self.g_random_transpose = Gtk.Entry()
+        cvbox.pack_start(gu.hig_label_widget(_("Random transpose:", True, True, 0),
             self.g_random_transpose, sizegroup))
         #
-        #self.g_statusbar = gtk.Statusbar()
+        #self.g_statusbar = Gtk.Statusbar()
         #self.toplevel_vbox.pack_start(self.g_statusbar, False)
         self.update_appwin()
     def proceed_if_changed(self):
         if not self.m_P.m_changed:
             return True
-        dialog = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION,
-              gtk.BUTTONS_YES_NO, _("You have unsaved data. Proceed anyway?"))
+        dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.QUESTION,
+              Gtk.ButtonsType.YES_NO, _("You have unsaved data. Proceed anyway?"))
         dialog.hide()
-        if dialog.run() == gtk.RESPONSE_YES:
+        if dialog.run() == Gtk.ResponseType.YES:
             dialog.destroy()
             return True
         dialog.destroy()
@@ -267,27 +266,27 @@ class MainWin(gtk.Window):
             self.m_P._idx = 0
 
         if self.m_P.header.module not in ('idbyname', 'chord', 'chordvoicing'):
-                dialog = gtk.MessageDialog(self, gtk.DIALOG_MODAL,
-                    gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
+                dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL,
+                    Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE,
                     _("The exercise module '%s' is not supported yet. Cannot edit this file.") % c)
                 dialog.run()
                 dialog.destroy()
                 self.m_P = EditorLessonfile()
         self.update_appwin()
     def file_open_cb(self, *v):
-        dialog = gtk.FileChooserDialog(_("Open..."), self,
-                                   gtk.FILE_CHOOSER_ACTION_OPEN,
-                                   (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                    gtk.STOCK_OPEN, gtk.RESPONSE_OK))
-        dialog.set_default_response(gtk.RESPONSE_OK)
-        if dialog.run() == gtk.RESPONSE_OK:
+        dialog = Gtk.FileChooserDialog(_("Open..."), self,
+                                   Gtk.FileChooserAction.OPEN,
+                                   (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                                    Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+        dialog.set_default_response(Gtk.ResponseType.OK)
+        if dialog.run() == Gtk.ResponseType.OK:
             filename = gu.decode_filename(dialog.get_filename())
             try:
                 self.load_file(filename)
             except Exception, e:
                 dialog.destroy()
-                m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
-                        gtk.BUTTONS_CLOSE,
+                m = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.ERROR,
+                        Gtk.ButtonsType.CLOSE,
                         _("Loading file '%(filename)s' failed: %(msg)s") %
                             {'filename': filename, 'msg': e})
                 m.run()
@@ -302,26 +301,26 @@ class MainWin(gtk.Window):
             self.update_appwin()
     def file_save_as_cb(self, *v):
         self.store_data_from_ui()
-        dialog = gtk.FileChooserDialog(_("Save as..."), self,
-                                  gtk.FILE_CHOOSER_ACTION_SAVE,
-                                   (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                    gtk.STOCK_SAVE, gtk.RESPONSE_OK))
-        dialog.set_default_response(gtk.RESPONSE_OK)
+        dialog = Gtk.FileChooserDialog(_("Save as..."), self,
+                                  Gtk.FileChooserAction.SAVE,
+                                   (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                                    Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+        dialog.set_default_response(Gtk.ResponseType.OK)
 
-        if dialog.run() == gtk.RESPONSE_OK:
+        if dialog.run() == Gtk.ResponseType.OK:
             self.m_P.m_filename = gu.decode_filename(dialog.get_filename())
             self.save_file()
         dialog.destroy()
     def file_save_cb(self, *v):
         self.store_data_from_ui()
         if self.m_P.m_filename is None:
-            dialog = gtk.FileChooserDialog(_("Save..."), self,
-                                   gtk.FILE_CHOOSER_ACTION_SAVE,
-                                   (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                    gtk.STOCK_SAVE, gtk.RESPONSE_OK))
-            dialog.set_default_response(gtk.RESPONSE_OK)
+            dialog = Gtk.FileChooserDialog(_("Save..."), self,
+                                   Gtk.FileChooserAction.SAVE,
+                                   (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                                    Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+            dialog.set_default_response(Gtk.ResponseType.OK)
 
-            if dialog.run() == gtk.RESPONSE_OK:
+            if dialog.run() == Gtk.ResponseType.OK:
                 self.m_P.m_filename = gu.decode_filename(dialog.get_filename())
             dialog.destroy()
         if self.m_P.m_filename:
@@ -352,7 +351,7 @@ class MainWin(gtk.Window):
 
     def quit_cb(self, *v):
         if self.proceed_if_changed():
-            gtk.main_quit()
+            Gtk.main_quit()
     def help_cb(self, *v):
         if not self.g_help_window:
             self.g_help_window = HelpWindow(self)
@@ -371,8 +370,8 @@ edit your hand written lesson files with this program.</p>
         else:
             self.g_help_window.present()
     def about_cb(self, *v):
-        dialog = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
-            gtk.BUTTONS_CLOSE, "GNU Solfege lesson file editor %s\nCopyright (C) 2004, 2005 Tom Cato Amundsen <tca@gnu.org>" % app_version)
+        dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.INFO,
+            Gtk.ButtonsType.CLOSE, "GNU Solfege lesson file editor %s\nCopyright (C) 2004, 2005 Tom Cato Amundsen <tca@gnu.org>" % app_version)
         dialog.run()
         dialog.destroy()
     def goto_first_question_cb(self, *v):
@@ -467,19 +466,19 @@ class UIManagerMainWin(MainWin):
     def __init__(self, datadir):
         MainWin.__init__(self, datadir)
     def create_menu_and_toolbar(self):
-        self.window_ag = gtk.ActionGroup('WindowActions')
-        self.lessonfile_ag = gtk.ActionGroup('LessonfileActions')
+        self.window_ag = Gtk.ActionGroup('WindowActions')
+        self.lessonfile_ag = Gtk.ActionGroup('LessonfileActions')
 
         self.window_ag.add_actions(fix_actions(window_actions, self))
         self.lessonfile_ag.add_actions(fix_actions(lessonfile_actions, self))
-        self.ui = gtk.UIManager()
+        self.ui = Gtk.UIManager()
         self.ui.insert_action_group(self.window_ag, 0)
         self.ui.insert_action_group(self.lessonfile_ag, 1)
         self.ui.add_ui_from_string(ui_string)
         self.add_accel_group(self.ui.get_accel_group())
-        self.toplevel_vbox.pack_start(self.ui.get_widget('/Menubar'), False)
-        self.ui.get_widget('/Toolbar').set_style(gtk.TOOLBAR_ICONS)
-        self.toplevel_vbox.pack_start(self.ui.get_widget('/Toolbar'), False)
+        self.toplevel_vbox.pack_start(self.ui.get_widget('/Menubar', True, True, 0), False)
+        self.ui.get_widget('/Toolbar').set_style(Gtk.TOOLBAR_ICONS)
+        self.toplevel_vbox.pack_start(self.ui.get_widget('/Toolbar', True, True, 0), False)
 
 def main(datadir):
     mpd.engravers.fetadir = os.path.join(datadir, "feta")
@@ -487,4 +486,4 @@ def main(datadir):
     if len(sys.argv) == 2:
         w.load_file(sys.argv[1])
     w.show()
-    gtk.main()
+    Gtk.main()
