@@ -48,43 +48,45 @@ from solfege import winlang
 from solfege import buildinfo
 from solfege.esel import FrontPage, TestsView, SearchView
 
-import gtk
+from gi.repository import Gtk
+from gi.repository import Gdk
+
 from solfege import utils
 from solfege import i18n
 
-class SplashWin(gtk.Window):
+class SplashWin(Gtk.Window):
     def __init__(self):
-        gtk.Window.__init__(self, gtk.WINDOW_POPUP)
-        self.set_position(gtk.WIN_POS_CENTER)
+        Gtk.Window.__init__(self, Gtk.WindowType.POPUP)
+        self.set_position(Gtk.WindowPosition.CENTER)
         self.set_resizable(True)
-        frame = gtk.Frame()
-        frame.set_shadow_type(gtk.SHADOW_OUT)
+        frame = Gtk.Frame()
+        frame.set_shadow_type(Gtk.ShadowType.OUT)
         self.add(frame)
-        vbox = gtk.VBox()
+        vbox = Gtk.VBox()
         vbox.set_border_width(20)
         frame.add(vbox)
-        l = gtk.Label(_("Starting GNU Solfege %s") % buildinfo.VERSION_STRING)
+        l = Gtk.Label(label=_("Starting GNU Solfege %s") % buildinfo.VERSION_STRING)
         l.set_name("Heading1")
-        vbox.pack_start(l)
-        l = gtk.Label("http://www.solfege.org")
-        vbox.pack_start(l)
-        self.g_infolabel = gtk.Label('')
-        vbox.pack_start(self.g_infolabel)
+        vbox.pack_start(l, True, True, 0)
+        l = Gtk.Label(label="http://www.solfege.org")
+        vbox.pack_start(l, True, True, 0)
+        self.g_infolabel = Gtk.Label(label='')
+        vbox.pack_start(self.g_infolabel, True, True, 0)
         self.show_all()
     def show_progress(self, txt):
         self.g_infolabel.set_text(txt)
-        while gtk.events_pending():
-            gtk.main_iteration(0)
+        while Gtk.events_pending():
+            Gtk.main_iteration()
 
 from solfege.configwindow import ConfigWindow
 from solfege.profilemanager import ChangeProfileDialog
 from solfege import gu
 from solfege import cfg
 from solfege import mpd
-from solfege import stock
 from solfege import lessonfile
 from solfege import download_pyalsa
 from solfege import statistics
+from solfege import stock
 
 
 from solfege import frontpage
@@ -93,12 +95,12 @@ from solfege.trainingsetdlg import TrainingSetDialog
 from solfege.practisesheetdlg import PractiseSheetDialog
 from solfege import filesystem
 
-class MusicViewerWindow(gtk.Dialog):
+class MusicViewerWindow(Gtk.Dialog):
     def __init__(self):
-        gtk.Dialog.__init__(self)
+        Gtk.Dialog.__init__(self)
         self.set_default_size(500, 300)
         self.g_music_displayer = mpd.MusicDisplayer()
-        self.vbox.pack_start(self.g_music_displayer)
+        self.vbox.pack_start(self.g_music_displayer, True, True, 0)
         b = gu.bButton(self.action_area, _("Close"), solfege.win.close_musicviewer)
         b.grab_focus()
         self.connect('destroy', solfege.win.close_musicviewer)
@@ -108,18 +110,16 @@ class MusicViewerWindow(gtk.Dialog):
         self.g_music_displayer.display(music, fontsize)
 
 
-class MainWin(gtk.Window, cfg.ConfigUtils):
+class MainWin(Gtk.Window, cfg.ConfigUtils):
     default_front_page = os.path.join(lessonfile.exercises_dir, 'learningtree.txt')
     debug_front_page = os.path.join(lessonfile.exercises_dir, 'debugtree.txt')
     def __init__(self, options, datadir):
-        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
-        self._vbox = gtk.VBox()
+        Gtk.Window.__init__(self, Gtk.WindowType.TOPLEVEL)
+        self._vbox = Gtk.VBox()
         self._vbox.show()
         self.add(self._vbox)
         stock.SolfegeIconFactory(self, datadir)
-        gtk.settings_get_default().set_property('gtk-button-images', True)
-        pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
-        gtk.window_set_default_icon(pixbuf)
+        Gtk.Settings.get_default().set_property('gtk-button-images', True)
         cfg.ConfigUtils.__dict__['__init__'](self, 'mainwin')
         self.set_resizable(self.get_bool('gui/mainwin_user_resizeable'))
         self.add_watch('gui/mainwin_user_resizeable', lambda s: self.set_resizable(self.get_bool('gui/mainwin_user_resizeable')))
@@ -133,17 +133,17 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.g_path_info_dlg = None
         self.g_musicviewer_window = None
         self.m_history = []
-        self.g_ui_manager = gtk.UIManager()
+        self.g_ui_manager = Gtk.UIManager()
         self.m_action_groups = {
-            'Exit': gtk.ActionGroup('Exit'),
-            'NotExit': gtk.ActionGroup('NotExit'),
+            'Exit': Gtk.ActionGroup('Exit'),
+            'NotExit': Gtk.ActionGroup('NotExit'),
         }
         for a in self.m_action_groups.values():
             self.g_ui_manager.insert_action_group(a, 1)
         self.setup_menu()
-        self.main_box = gtk.VBox()
+        self.main_box = Gtk.VBox()
         self.main_box.show()
-        self._vbox.pack_start(self.main_box)
+        self._vbox.pack_start(self.main_box, True, True, 0)
     def get_view(self):
         """
         Return the view that is currently visible.
@@ -159,7 +159,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if self.m_viewer:
             self.get_view().hide()
         self.box_dict[name] = view
-        self.main_box.pack_start(self.box_dict[name])
+        self.main_box.pack_start(self.box_dict[name], True, True, 0)
         self.box_dict[name].show()
         self.m_viewer = name
     def show_view(self, name):
@@ -247,7 +247,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
           ('HelpOnline', None, _('_Mailing lists, web page etc.'), None, None,
             lambda o: solfege.app.handle_href('online-resources.html')),
           ('HelpDonate', None, _('_Donate'), None, None,
-            lambda o: solfege.app.handle_href('http://www.solfege.org/Main/Donate')),
+            lambda o: solfege.app.handle_href('http://www.solfege.org/donate/')),
           ('HelpReportingBugs', None, _('Reporting _bugs'), None, None,
             lambda o: solfege.app.handle_href('bug-reporting.html')),
           ('HelpAbout', 'gtk-about', None, None, None, self.show_about_window),
@@ -258,10 +258,10 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.g_ui_manager.add_ui_from_file("ui.xml")
 
         self.add_accel_group(self.g_ui_manager.get_accel_group())
-        hdlbox = gtk.HandleBox()
+        hdlbox = Gtk.HandleBox()
         hdlbox.show()
         hdlbox.add(self.g_ui_manager.get_widget('/Menubar'))
-        self._vbox.pack_start(hdlbox, False)
+        self._vbox.pack_start(hdlbox, False, False, 0)
         self.m_help_on_current_merge_id = None
     def create_frontpage_menu(self):
         """
@@ -322,26 +322,26 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.g_ui_manager.remove_ui(self.m_help_on_current_merge_id)
         self.m_help_on_current_merge_id = None
     def show_bug_reports(self, *v):
-        m = gtk.Dialog(_("Question"), self, 0)
-        m.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        m.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
-        vbox = gtk.VBox()
-        m.vbox.pack_start(vbox, False)
+        m = Gtk.Dialog(_("Question"), self, 0)
+        m.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        m.add_button(Gtk.STOCK_OK, Gtk.ResponseType.OK)
+        vbox = Gtk.VBox()
+        m.vbox.pack_start(vbox, False, False, 0)
         vbox.set_spacing(18)
         vbox.set_border_width(12)
-        l = gtk.Label(_("Please enter the email used when you submitted the bugs:"))
-        vbox.pack_start(l, False)
-        self.g_email = gtk.Entry()
+        l = Gtk.Label(label=_("Please enter the email used when you submitted the bugs:"))
+        vbox.pack_start(l, False, False, 0)
+        self.g_email = Gtk.Entry()
         m.action_area.get_children()[0].grab_default()
         self.g_email.set_activates_default(True)
-        vbox.pack_start(self.g_email, False)
+        vbox.pack_start(self.g_email, False, False, 0)
         m.show_all()
         ret = m.run()
         m.destroy()
-        if ret == gtk.RESPONSE_OK:
+        if ret == Gtk.ResponseType.OK:
             params = urllib.urlencode({
                     'pagename': 'SITS-Incoming/SearchBugs',
-                    'q': 'SITS-Incoming/"Submitter: %s"' % utils.mangle_email(self.g_email.get_text()),
+                    'q': 'SITS-Incoming/"Submitter: %s"' % utils.mangle_email(self.g_email.get_text().decode("utf-8")()),
                 })
             try:
                 webbrowser.open_new("http://www.solfege.org?%s" % params)
@@ -361,16 +361,16 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             text = text.decode(locale.getpreferredencoding(), 'replace')
         if not isinstance(secondary_text, unicode):
             secondary_text = secondary_text.decode(locale.getpreferredencoding(), 'replace')
-        m = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_CLOSE, text)
+        m = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL, Gtk.MessageType.ERROR,
+                              Gtk.ButtonsType.CLOSE, text)
         if secondary_text:
             m.format_secondary_text(secondary_text)
         m.run()
         m.destroy()
         if reshow_splash:
             solfege.splash_win.show()
-            while gtk.events_pending():
-                gtk.main_iteration(0)
+            while Gtk.events_pending():
+                Gtk.main_iteration()
     def display_error_message(self, msg, title=None, secondary_text=None):
         if solfege.splash_win and solfege.splash_win.props.visible:
             solfege.splash_win.hide()
@@ -379,8 +379,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             reshow_splash = False
         if not isinstance(msg, unicode):
             msg = msg.decode(locale.getpreferredencoding(), 'replace')
-        m = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_CLOSE, None)
+        m = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL, Gtk.MessageType.ERROR,
+                              Gtk.ButtonsType.CLOSE, None)
         m.set_markup(gu.escape(msg))
         if title:
             m.set_title(title)
@@ -390,54 +390,55 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         m.destroy()
         if reshow_splash:
             solfege.splash_win.show()
-            while gtk.events_pending():
-                gtk.main_iteration(0)
+            while Gtk.events_pending():
+                Gtk.main_iteration()
     def show_path_info(self, w):
         if not self.g_path_info_dlg:
-            self.g_path_info_dlg = gtk.Dialog(_("_File locations").replace("_", ""), self,
-                buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-            sc = gtk.ScrolledWindow()
-            sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
-            self.g_path_info_dlg.vbox.pack_start(sc, True, True)
+            self.g_path_info_dlg = Gtk.Dialog(_("_File locations").replace("_", ""), self,
+                buttons=(Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
+            sc = Gtk.ScrolledWindow()
+            sc.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.NEVER)
+            self.g_path_info_dlg.vbox.pack_start(sc, True, True, 0)
             #
             vbox = gu.hig_dlg_vbox()
             sc.add_with_viewport(vbox)
 
             box1, box2 = gu.hig_category_vbox(_("_File locations").replace("_", ""))
-            vbox.pack_start(box1)
-            sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+            vbox.pack_start(box1, True, True, 0)
+            sizegroup = Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL)
             # statistics.sqlite
             # win32 solfegerc
             # win32 langenviron.txt
-            box2.pack_start(gu.hig_label_widget(_("Solfege application data:"), gtk.Label(filesystem.app_data()), sizegroup))
-            box2.pack_start(gu.hig_label_widget(_("Solfege user data:"), gtk.Label(filesystem.user_data()), sizegroup))
-            box2.pack_start(gu.hig_label_widget(_("Solfege config file:"), gtk.Label(filesystem.rcfile()), sizegroup))
-            box2.pack_start(gu.hig_label_widget(_("Solfege installation directory:"), gtk.Label(os.getcwdu()), sizegroup))
-            box2.pack_start(gu.hig_label_widget(_("User manual in HTML format:"), gtk.Label(os.path.join(os.getcwdu(), "help")), sizegroup))
-            box2.pack_start(gu.hig_label_widget("gtk:", gtk.Label(str(gtk)), sizegroup))
-            box2.pack_start(gu.hig_label_widget("pyalsa:", gtk.Label(str(alsaseq)), sizegroup))
-            box2.pack_start(gu.hig_label_widget("PYTHONHOME", gtk.Label(os.environ.get('PYTHONHOME', 'Not defined')), sizegroup))
+            box2.pack_start(gu.hig_label_widget(_("Solfege application data:"), Gtk.Label(label=filesystem.app_data()), sizegroup), False, False, 0)
+            box2.pack_start(gu.hig_label_widget(_("Solfege user data:"), Gtk.Label(label=filesystem.user_data()), sizegroup), False, False, 0)
+            box2.pack_start(gu.hig_label_widget(_("Solfege config file:"), Gtk.Label(label=filesystem.rcfile()), sizegroup), False, False, 0)
+            box2.pack_start(gu.hig_label_widget(_("Solfege installation directory:"), Gtk.Label(label=os.getcwdu()), sizegroup), False, False, 0)
+            box2.pack_start(gu.hig_label_widget(_("User manual in HTML format:"), Gtk.Label(label=os.path.join(os.getcwdu(), "help")), sizegroup), False, False, 0)
+            box2.pack_start(gu.hig_label_widget("gtk:", Gtk.Label(label=str(Gtk)), sizegroup), False, False, 0)
+            box2.pack_start(gu.hig_label_widget("pyalsa:", Gtk.Label(label=str(alsaseq)), sizegroup), False, False, 0)
+            box2.pack_start(gu.hig_label_widget("PYTHONHOME", Gtk.Label(os.environ.get('PYTHONHOME', 'Not defined')), sizegroup), False, False, 0)
             self.g_path_info_dlg.show_all()
             def f(*w):
                 self.g_path_info_dlg.hide()
                 return True
             self.g_path_info_dlg.connect('response', f)
             self.g_path_info_dlg.connect('delete-event', f)
-            w, h = vbox.size_request()
-            sc.set_size_request(min(w + gu.hig_dlg_vbox.border_width * 2, gtk.gdk.screen_width()*0.9), h)
+            sc.set_size_request(min(vbox.size_request().width + gu.hig.SPACE_LARGE * 2,
+                                    Gdk.Screen.width() * 0.9),
+                                vbox.size_request().height)
     def setup_pyalsa(self, widget):
         download_pyalsa.download()
     def show_about_window(self, widget):
-        trans = _("SOLFEGETRANSLATORS")
-        if trans == 'SOLFEGETRANSLATORS':
-            trans = ""
-        pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
-        a = self.g_about_window = gtk.AboutDialog()
+        pixbuf = self.render_icon('solfege-icon', Gtk.IconSize.DIALOG)
+        a = self.g_about_window = Gtk.AboutDialog()
+        a.set_program_name("GNU Solfege")
         a.set_logo(pixbuf)
-        a.set_name("GNU Solfege")
         a.set_website("http://www.solfege.org")
         a.set_version(buildinfo.VERSION_STRING)
-        a.set_copyright("\n".join((solfege.application.solfege_copyright, solfege.application.warranty)))
+        a.set_copyright("Copyright (C) 2013 Tom Cato Amundsen and others")
+        a.set_license("\n".join((solfege.application.solfege_copyright, solfege.application.warranty)))
+        # Using set_license_type causes the app to print warnings.
+        #a.set_license_type(Gtk.License.GPL_3_0)
         a.set_authors(["Tom Cato Amundsen",
               'Giovanni Chierico %s' % _("(some lessonfiles)"),
               'Michael Becker %s' % _("(some lessonfiles)"),
@@ -453,7 +454,10 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         a.set_documenters(["Tom Cato Amundsen",
                 "Tom Eykens",
                 ])
-        a.set_translator_credits(_("SOLFEGETRANSLATORS"))
+        if _("SOLFEGETRANSLATORS") == 'SOLFEGETRANSLATORS':
+            a.set_translator_credits(None)
+        else:
+            a.set_translator_credits(_("SOLFEGETRANSLATORS"))
         self.g_about_window.run()
         self.g_about_window.destroy()
     def do_tree_editor(self, *v):
@@ -525,7 +529,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             pass
         for filename in lessonfile.infocache.frontpage.iter_old_format_files():
             gu.dialog_ok(_("Cannot load front page file"), None,
-                _("The file «%s» is saved in an old file format. The file can be converted by editing and saving it with an older version of Solfege. Versions from 3.16.0 to 3.20.4 should do the job.") % filename)
+                _(u"The file «%s» is saved in an old file format. The file can be converted by editing and saving it with an older version of Solfege. Versions from 3.16.0 to 3.20.4 should do the job.") % filename)
     def activate_exercise(self, module, urlobj=None):
         self.show_view(module)
         # We need this test because not all exercises use a notebook.
@@ -645,9 +649,9 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             view = self.get_view()
         except KeyError:
             return
-        if (event.type == gtk.gdk.KEY_PRESS
-            and event.state == gtk.gdk.MOD1_MASK # Alt key
-            and event.keyval in (gtk.keysyms.KP_Left, gtk.keysyms.Left)
+        if (event.type == Gdk.EventType.KEY_PRESS
+            and event.get_state() & Gdk.ModifierType.MOD1_MASK == Gdk.ModifierType.MOD1_MASK# Alt key
+            and event.keyval in (Gdk.KEY_KP_Left, Gdk.KEY_Left)
             and self.m_history
             and not solfege.app.m_test_mode):
             obj, page = self.m_history[-1]
@@ -662,7 +666,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         view.on_key_press_event(widget, event)
     def open_profile_manager(self, widget=None):
         p = ChangeProfileDialog(solfege.app.m_options.profile)
-        if p.run() == gtk.RESPONSE_ACCEPT:
+        if p.run() == Gtk.ResponseType.ACCEPT:
             prof = p.get_profile()
         else:
             # The user presses cancel. This will use the same profile as
@@ -698,7 +702,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 break
         if can_quit:
             solfege.app.quit_program()
-            gtk.main_quit()
+            Gtk.main_quit()
         else:
             return True
     def display_in_musicviewer(self, music):
@@ -721,8 +725,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def exit_test_mode(self):
         solfege.app.m_test_mode = False
         self.m_action_groups['NotExit'].set_sensitive(True)
-        box = gtk.VBox()
-        self.get_view().g_notebook.insert_page(box, gtk.Label(_("Practise")), 0)
+        box = Gtk.VBox()
+        self.get_view().g_notebook.insert_page(box, Gtk.Label(label=_("Practise")), 0)
         self.g.reparent(box)
         self.get_view().g_notebook.show()
         self.get_view().g_notebook.get_nth_page(0).show()
