@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,10 @@
 from __future__ import absolute_import
 import operator
 
-import gtk
-import pango
+from gi.repository import Gtk
+from gi.repository import Pango
+from gi.repository import GdkPixbuf
+import cairo
 
 from solfege.mpd import const
 from solfege.mpd import elems
@@ -87,38 +89,41 @@ class ClefEngraver(Engraver):
     def __init__(self, clef):
         Engraver.__init__(self)
         self.m_clef = clef
+
     def get_width(self):
         return 25
-    def engrave(self, widget, gc, staff_yoffset):
-        widget.window.draw_pixbuf(gc,
-            gtk.gdk.pixbuf_new_from_file(fetadir+'/feta%i-clefs-%s.xpm' \
-                % (self.m_fontsize, self.m_clef.get_symbol())),
-            0, 0, self.m_xpos,
-            dimentions[self.m_fontsize].clef_yoffset[self.m_clef.get_symbol()] - self.m_clef.get_stafflinepos() * dimentions[self.m_fontsize].linespacing\
-                + staff_yoffset)
+
+    def engrave(self, ct, staff_yoffset):
+        ims = cairo.ImageSurface.create_from_png(
+            fetadir + '/feta%i-clefs-%s.png'
+            % (self.m_fontsize, self.m_clef.get_symbol()))
+        ct.set_source_surface(
+            ims, self.m_xpos,
+            dimentions[self.m_fontsize].clef_yoffset[self.m_clef.get_symbol()]
+            - self.m_clef.get_stafflinepos() * dimentions[self.m_fontsize].linespacing
+            + staff_yoffset)
+        ct.paint()
+
 
 class TimeSignatureEngraver(Engraver):
     def __init__(self, timesig):
         Engraver.__init__(self)
         self.m_timesig = timesig
-    def engrave(self, widget, gc, staff_yoffset):
+
+    def engrave(self, ct, staff_yoffset):
         x = 0
         for idx in range(len(str(self.m_timesig.m_num))):
             n = str(self.m_timesig.m_num)[idx]
-            widget.window.draw_pixbuf(gc,
-                gtk.gdk.pixbuf_new_from_file(fetadir+'/feta%i-number-%s.xpm' \
-                    % (self.m_fontsize, n)),
-                0, 0,
-                self.m_xpos+x, staff_yoffset-12)
-            x  += 10
+            ims = cairo.ImageSurface.create_from_png(
+                fetadir + '/feta%i-number-%s.png' % (self.m_fontsize, n))
+            ct.set_source_surface(ims, self.m_xpos + x, staff_yoffset - 12)
+            x += 10
         x = 0
         for idx in range(len(str(self.m_timesig.m_den))):
             n = str(self.m_timesig.m_den)[idx]
-            widget.window.draw_pixbuf(gc,
-                gtk.gdk.pixbuf_new_from_file(fetadir+'/feta%i-number-%s.xpm' %\
-                    (self.m_fontsize, n)),
-                0, 0,
-                self.m_xpos+x, staff_yoffset+3)
+            ims = cairo.ImageSurface.create_from_png(
+                fetadir + '/feta%i-number-%s.png' % (self.m_fontsize, n))
+            ct.set_source_surface(ims, self.m_xpos + x, staff_yoffset + 3)
             x += 10
     def __str__(self):
         return '(TimeSignatureEngraver:%i/%i)' % (self.m_timesig.m_den,
@@ -130,14 +135,17 @@ class TieEngraver(Engraver):
         Engraver.__init__(self)
         self.m_note1 = note1
         self.m_note2 = note2
-    def engrave(self, widget, gc, staff_yoffset):
+
+    def engrave(self, ct, staff_yoffset):
         dim = dimentions[self.m_fontsize]
-        w = self.m_note2.m_xpos - self.m_note1.m_xpos - dim.xshift * 1.3
-        h = 8
-        widget.window.draw_arc(gc, False,
-              int(self.m_note1.m_xpos + dim.xshift * 1.2),
-              int(staff_yoffset + dim.linespacing*(self.m_note2.m_ypos-1)/2 + 3),
-              int(w), h, 64 * 180, 64*180)
+        p1 = self.m_note1.m_xpos + dim.xshift
+        p2 = self.m_note2.m_xpos
+        w = p2 - p1 - dim.xshift
+        w = max(w, dim.xshift)
+        ct.arc((p1 + (p2 - p1) / 2),
+               int(staff_yoffset + dim.linespacing * (self.m_note2.m_ypos-1) / 2 + 3),
+               int(w), 30 * 3.14 / 180, 150 * 3.14 / 180)
+        ct.stroke()
 
 
 class AccidentalsEngraver(Engraver):
@@ -152,20 +160,19 @@ class AccidentalsEngraver(Engraver):
         self.m_usize = reduce(operator.__add__, map(f, accs.values())) + len(accs)
     def get_width(self):
         return self.m_usize
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         x = 0
         for y in self.m_accs:
             for acc in self.m_accs[y]:
-                widget.window.draw_pixbuf(gc,
-                    gtk.gdk.pixbuf_new_from_file(
-                        fetadir+'/feta%i-accidentals-%i.xpm' % \
-                            (self.m_fontsize, acc)),
-                        0, 0,
-                        self.m_xpos + x,
+                ims = cairo.ImageSurface.create_from_png(
+                        fetadir+'/feta%i-accidentals-%i.png' % \
+                            (self.m_fontsize, acc))
+                ct.set_source_surface(ims, self.m_xpos + x,
                         int(accidental_y_offset[acc] + staff_yoffset
                          + dimentions[self.m_fontsize].linespacing*y/2
                          + accidental_y_offset[acc]))
                 x += dimentions[self.m_fontsize].accidental_widths[acc] + 1
+        ct.paint()
 
 
 class KeySignatureEngraver(Engraver):
@@ -178,7 +185,7 @@ class KeySignatureEngraver(Engraver):
         #FIXME this value depends on the fontsize, and on what kind of
         #accidental we are drawing. A sharp is wider that a flat.
         return len(self.m_accidentals) * 10 + len(self.m_old_accidentals) * 6 + 8
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         x = 0
         dolist = []
         # natural signs
@@ -204,15 +211,13 @@ class KeySignatureEngraver(Engraver):
             #FIXME see FIXME msg in .get_width
             x += 10
         for ktype, x, ypos in dolist:
-            widget.window.draw_pixbuf(gc,
-                gtk.gdk.pixbuf_new_from_file(
-                    fetadir+'/feta%i-accidentals-%i.xpm' \
-                    % (self.m_fontsize, ktype)),
-                0, 0,
-                self.m_xpos + x,
+            ims = cairo.ImageSurface.create_from_png(
+                fetadir + '/feta%i-accidentals-%i.png' % (self.m_fontsize, ktype))
+            ct.set_source_surface(ims, self.m_xpos + x,
                 int(accidental_y_offset[ktype] + staff_yoffset
-                     + dimentions[self.m_fontsize].linespacing*ypos/2
+                     + dimentions[self.m_fontsize].linespacing * ypos / 2
                      + accidental_y_offset[ktype]))
+            ct.paint()
 
 
 class NoteheadEngraver(Engraver):
@@ -235,26 +240,25 @@ class NoteheadEngraver(Engraver):
         self.m_numdots = duration.m_dots
         self.m_ypos = ypos
         self.m_midi_int = int(musicalpitch)
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         dim = dimentions[self.m_fontsize]
-        # Have to adjust holenotes a little to the left to be on the middle
+        # Have to adjust wholenotes a little to the left to be on the middle
         # of the ledger line.
         if self.m_head == 0:
             xx = -2
         else:
             xx = 0
-        widget.window.draw_pixbuf(gc,
-            gtk.gdk.pixbuf_new_from_file(fetadir+'/feta%i-noteheads-%i.xpm' \
-                % (self.m_fontsize, self.m_head)),
-            0, 0,
-            self.m_xpos + self.m_shift * dim.xshift + xx,
-            int(staff_yoffset + dim.linespacing*self.m_ypos/2 - 4))
+        ims = cairo.ImageSurface.create_from_png(
+            fetadir + '/feta%i-noteheads-%i.png' % (self.m_fontsize, self.m_head))
+        ct.set_source_surface(ims, self.m_xpos + self.m_shift * dim.xshift + xx,
+            int(staff_yoffset + dim.linespacing * self.m_ypos / 2 - 4))
+        ct.paint()
         for n in range(self.m_numdots):
-            widget.window.draw_pixbuf(gc,
-                gtk.gdk.pixbuf_new_from_file(fetadir+'/feta20-dots-dot.xpm'),
-                0, 0,
-                int(self.m_xpos+dim.xshift*(self.m_shift+1.5+n/2.0)),
-                -3 + staff_yoffset + dim.linespacing*self.m_ypos/2)
+            ims = cairo.ImageSurface.create_from_png(
+                fetadir + '/feta20-dots-dot.png')
+            ct.set_source_surface(ims, int(self.m_xpos + dim.xshift * (self.m_shift + 1.5 + n/2.0)),
+                -3 + staff_yoffset + dim.linespacing * self.m_ypos / 2)
+        ct.paint()
     def __str__(self):
         try:
             xpos = self.m_xpos
@@ -277,10 +281,11 @@ class BarlineEngraver(Engraver):
         self.m_type = barline_type
     def get_width(self):
         return 8
-    def engrave(self, widget, gc, staff_yoffset):
+
+    def engrave(self, ct, staff_yoffset):
         dim = dimentions[self.m_fontsize]
-        widget.window.draw_line(gc, self.m_xpos, staff_yoffset - dim.linespacing*2,
-                                    self.m_xpos, staff_yoffset + dim.linespacing*2)
+        ct.move_to(self.m_xpos + 0.5, staff_yoffset - dim.linespacing * 2)
+        ct.line_to(self.m_xpos + 0.5, staff_yoffset + dim.linespacing * 2)
 
 class TupletEngraver(Engraver):
     def __init__(self, ratio, direction):
@@ -307,7 +312,7 @@ class TupletEngraver(Engraver):
         self.m_xpos2 = self.m_stems[-1].m_xpos + 2
         if self.m_stems[-1].m_stemdir == const.DOWN:
             self.m_xpos2 += dim.xshift
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         self.do_layout()
         dim = dimentions[self.m_fontsize]
         m = (self.m_xpos1 + self.m_xpos2) / 2
@@ -319,21 +324,27 @@ class TupletEngraver(Engraver):
             y = max(staff_yoffset + dim.linespacing * 5,
                     staff_yoffset + self.m_b * dim.linespacing /2)
             d = -1
+        self.m_xpos1 += 0.5
+        self.m_xpos2 += 0.5
         if len(self.m_stems) > 1:
             # left horiz
-            widget.window.draw_line(gc, self.m_xpos1, y, m - 6, y)
+            ct.move_to(self.m_xpos1, y)
+            ct.line_to(m - 6, y)
             # right horiz
-            widget.window.draw_line(gc, m + 6, y, self.m_xpos2, y)
+            ct.move_to(m + 6, y)
+            ct.line_to(self.m_xpos2, y)
             # left vertic
-            widget.window.draw_line(gc, self.m_xpos1, y, self.m_xpos1, y+5*d)
+            ct.move_to(self.m_xpos1, y)
+            ct.line_to(self.m_xpos1, y + 5 * d)
             # right vertic
-            widget.window.draw_line(gc, self.m_xpos2, y, self.m_xpos2, y+5*d)
-        cc = widget.create_pango_context()
-        ll = pango.Layout(cc)
-        ll.set_text(str(self.m_ratio.m_den))
-        ll.set_font_description(pango.FontDescription("Sans 10"))
-        sx, sy = [i / pango.SCALE for i in ll.get_size()]
-        widget.window.draw_layout(gc, m - sx / 2, y - sy / 2, ll)
+            ct.move_to(self.m_xpos2, y)
+            ct.line_to(self.m_xpos2, y + 5 * d)
+            ct.stroke()
+        ct.select_font_face("Sans 10")
+        tt = ct.text_extents(str(self.m_ratio.m_den))
+        ct.move_to(m - tt[2] / 2, y + tt[3] / 2)
+        ct.show_text(str(self.m_ratio.m_den))
+        ct.stroke()
 
 
 class BeamEngraver(Engraver):
@@ -386,7 +397,7 @@ class BeamEngraver(Engraver):
         for e in self.m_stems:
             e.m_stemdir = stemdir
         self.m_stemdir = stemdir
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         # Do nothing if there are only one stem in the beam. Usually this
         # does not happen, but it is possible that a lesson file want to
         # display only the first note in some music, and that note is beamed.
@@ -408,21 +419,21 @@ class BeamEngraver(Engraver):
         beamw = 3
         beaml = 10
         for y in range(beamw):
-            widget.window.draw_line(gc, x1, y1 + y*d, x2, y1 + y*d)
+            ct.move_to(x1, y1 + y*d)
+            ct.line_to(x2, y1 + y*d)
         for stem in self.m_stems:
             stem._beam_done = 8
-        def short_beam(stem, xdir, beamnum, d=d, widget=widget, beamw=beamw, gc=gc, beaml=beaml, y1=y1):
+        def short_beam(stem, xdir, beamnum, d=d, beamw=beamw, beaml=beaml, y1=y1):
             for y in range(beamw):
-                widget.window.draw_line(gc, stem.m_xpos, y1 + y + beamnum*d*beamw*2,
-                       stem.m_xpos + xdir*beaml, y1 + y + beamnum*d*beamw*2)
+                ct.move_to(stem.m_xpos, y1 + y + beamnum * d * beamw * 2)
+                ct.line_to(stem.m_xpos + xdir * beaml, y1 + y + beamnum * d * beamw * 2)
         for nl, yc in ((16, 0), (32, 1), (64, 2)):
             for i in range(len(self.m_stems)-1):
                 if self.m_stems[i].m_duration.m_nh >= nl \
                         and self.m_stems[i+1].m_duration.m_nh >= nl:
                     for y in range(beamw):
-                        widget.window.draw_line(gc,
-                            self.m_stems[i].m_xpos, d*beamw*yc*2 + y1 + y + d*beamw*2,
-                            self.m_stems[i+1].m_xpos, d*beamw*yc*2 + y1 + y + d*beamw*2)
+                        ct.move_to(self.m_stems[i].m_xpos, d*beamw*yc*2 + y1 + y + d*beamw*2)
+                        ct.line_to(self.m_stems[i+1].m_xpos, d*beamw*yc*2 + y1 + y + d*beamw*2)
                     self.m_stems[i]._beam_done = nl
                     self.m_stems[i+1]._beam_done = nl
                 if self.m_stems[i].m_duration.m_nh >= nl \
@@ -441,6 +452,7 @@ class BeamEngraver(Engraver):
             if self.m_stems[-1].m_duration.m_nh >= nl \
                     and self.m_stems[-1]._beam_done <= nl/2:
                 short_beam(self.m_stems[-1], -1, yc+1, d)
+        ct.stroke()
 
 
 class StemEngraver(Engraver):
@@ -488,27 +500,23 @@ class StemEngraver(Engraver):
             self.m_xpos += dimentions[self.m_fontsize].xshift
             for n in self.m_noteheads:
                 n.m_xpos += dimentions[self.m_fontsize].xshift
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         if self.m_duration.m_nh < 2:
             return
         # draw flags
         if not self.m_is_beamed and self.m_duration.m_nh > 4:
             if self.m_stemdir == const.UP:
-                widget.window.draw_pixbuf(gc,
-                    gtk.gdk.pixbuf_new_from_file(
-                        fetadir+'/feta%i-flags-%s.xpm' % \
-                            (self.m_fontsize, self.get_flag(const.UP))),
-                    0, 0,
-                    self.m_xpos,
+                ims = cairo.ImageSurface.create_from_png(
+                    fetadir + '/feta%i-flags-%s.png' %
+                    (self.m_fontsize, self.get_flag(const.UP)))
+                ct.set_source_surface(ims, self.m_xpos,
                     int(staff_yoffset - self.m_stemlen + self.m_yposes[0] * dim.linespacing/2))
             else:
-                widget.window.draw_pixbuf(gc,
-                    gtk.gdk.pixbuf_new_from_file(
-                        fetadir+'/feta%i-flags-%s.xpm' \
-                        % (self.m_fontsize, self.get_flag(const.DOWN))),
-                    0, 0,
-                    self.m_xpos,
+                ims = cairo.ImageSurface.create_from_png(
+                    fetadir + '/feta%i-flags-%s.png'
+                    % (self.m_fontsize, self.get_flag(const.DOWN)))
+                ct.set_source_surface(ims, self.m_xpos,
                     int(staff_yoffset + 4 + self.m_yposes[-1] * dim.linespacing/2))
         # draw stem
         if self.m_stemdir == const.DOWN:
@@ -523,11 +531,8 @@ class StemEngraver(Engraver):
             else:
                 yroot = self.m_yposes[0] - 6
             ytop = self.m_yposes[-1]
-        widget.window.draw_line(gc,
-                     self.m_xpos,
-                     ytop * dim.linespacing/2 + staff_yoffset,
-                     self.m_xpos,
-                     yroot * dim.linespacing/2 + staff_yoffset)
+        ct.move_to(self.m_xpos, ytop * dim.linespacing / 2 + staff_yoffset)
+        ct.line_to(self.m_xpos, yroot * dim.linespacing / 2 + staff_yoffset)
     def get_flag(self, stemdir):
         assert self.m_duration.m_nh > 4
         return {8:{const.UP: FLAG_8UP,
@@ -568,6 +573,8 @@ class StemEngraver(Engraver):
             if last_ypos is not None:
                 if abs(last_ypos - head.m_ypos) == 1 and heads[idx - 1].m_shift == 0:
                     head.m_shift = 1 if self.m_stemdir == const.UP else -1
+                elif last_ypos == head.m_ypos:
+                    head.m_shift = 1 if self.m_stemdir == const.UP else -1
             last_ypos = head.m_ypos
     def __str__(self):
         return "(StemEngraver)"
@@ -586,18 +593,17 @@ class LedgerLineEngraver(Engraver):
         self.m_xpos = 0
         self.m_up = up
         self.m_down = down
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         if self.m_up:
-            for y in range(self.m_up):
-                widget.window.draw_line(gc,
-                    self.m_xpos+dim.ledger_left, (-y-3) * dim.linespacing + staff_yoffset,
-                    self.m_xpos+dim.ledger_right, (-y-3) * dim.linespacing + staff_yoffset)
+            for y in range(self.m_up):#FIXME use rel_line_to
+                ct.move_to(self.m_xpos+dim.ledger_left, (-y-3) * dim.linespacing + staff_yoffset)
+                ct.line_to(self.m_xpos+dim.ledger_right, (-y-3) * dim.linespacing + staff_yoffset)
         if self.m_down:
             for y in range(self.m_down):
-                widget.window.draw_line(gc,
-                    self.m_xpos+dim.ledger_left, (y+3) * dim.linespacing + staff_yoffset,
-                    self.m_xpos+dim.ledger_right, (y+3) * dim.linespacing + staff_yoffset)
+                ct.move_to(self.m_xpos+dim.ledger_left, (y+3) * dim.linespacing + staff_yoffset)
+                ct.line_to(self.m_xpos+dim.ledger_right, (y+3) * dim.linespacing + staff_yoffset)
+        ct.stroke()
     def __str__(self):
         return "(LedgerLineEngraver xpos:%i, updown%i%i" % (
             self.m_xpos, self.m_up, self.m_down)
@@ -612,27 +618,29 @@ class RestEngraver(Engraver):
         self.m_ypos = ypos
         self.m_dots = dur.m_dots
         self.m_type = {1: 0, 2: 1, 4: 2, 8: 3, 16: 4, 32: 5, 64: 6}[dur.m_nh]
+
     def get_width(self):
         #FIXME write me!
         return 20
-    def engrave(self, widget, gc, staff_yoffset):
+
+    def engrave(self, ct, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         if self.m_type == 0:
-            my = dim.linespacing/2
+            my = dim.linespacing / 2
         else:
             my = 0
-        widget.window.draw_pixbuf(gc,
-            gtk.gdk.pixbuf_new_from_file(fetadir+'/feta%i-rests-%i.xpm' \
-                % (self.m_fontsize, self.m_type)),
-            0, 0,
-            self.m_xpos,
-            int(staff_yoffset - my + dim.linespacing*self.m_ypos/2 - 4))
+        ims = cairo.ImageSurface.create_from_png(
+            fetadir+'/feta%i-rests-%i.png' % (self.m_fontsize, self.m_type))
+        ct.set_source_surface(
+            ims, self.m_xpos,
+            int(staff_yoffset - my + dim.linespacing*self.m_ypos / 2 - 4))
+        ct.paint()
         for n in range(self.m_dots):
-            widget.window.draw_pixbuf(gc,
-                gtk.gdk.pixbuf_new_from_file(fetadir+'/feta20-dots-dot.xpm'),
-                0, 0,
+            ims = cairo.ImageSurface.create_from_png(fetadir+'/feta20-dots-dot.xpm')
+            ct.set_source_surface(
                 int(self.m_xpos+dim.xshift*(1.5+n/2.0)),
                 -3 + staff_yoffset + dim.linespacing*self.m_ypos/2)
+            ct.paint()
     def __str__(self):
         return "(RestEngraver)"
 
@@ -641,7 +649,7 @@ class SkipEngraver(Engraver):
     def __init__(self, duration):
         Engraver.__init__(self)
         self.m_duration = duration
-    def engrave(self, widget, gc, staff_yoffset):
+    def engrave(self, ct, staff_yoffset):
         pass
 
 
